use derive_more::{Deref, Display, From, Into, with_trait::Display as TDisplay};
use serde::{Deserialize, Deserializer, Serialize, Serializer, de::Error as DeError};
use std::fmt::{Debug, Formatter, Result as FmtResult};
use strum_macros::{Display as SMDisplay, EnumString, VariantNames};

#[derive(
    Copy,
    Clone,
    PartialEq,
    Eq,
    PartialOrd,
    Ord,
    Debug,
    EnumString,
    SMDisplay,
    VariantNames,
    Serialize,
    Deserialize,
)]
#[strum(serialize_all = "title_case")]
pub enum TargetPointerWidth {
    #[strum(serialize = "32 Bit")]
    W32Bit,
    #[strum(serialize = "64 Bit")]
    W64Bit,
}

#[derive(Copy, Clone, PartialEq, Eq, PartialOrd, Ord, Deref, From, Into, Display)]
#[display("{_0}")]
pub struct FmtOSArchitecture(pub TargetPointerWidth);

impl FmtOSArchitecture {
    pub const W32_BIT: Self = Self(TargetPointerWidth::W32Bit);
    pub const W64_BIT: Self = Self(TargetPointerWidth::W64Bit);
}

impl Debug for FmtOSArchitecture {
    fn fmt(&self, f: &mut Formatter<'_>) -> FmtResult {
        TDisplay::fmt(self, f)
    }
}

impl Serialize for FmtOSArchitecture {
    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
    where
        S: Serializer,
    {
        serializer.serialize_str(&self.0.to_string())
    }
}

impl<'de> Deserialize<'de> for FmtOSArchitecture {
    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
    where
        D: Deserializer<'de>,
    {
        let s = String::deserialize(deserializer)?;
        s.parse::<TargetPointerWidth>()
            .map(FmtOSArchitecture)
            .map_err(|_| DeError::custom(format!("Invalid architecture: {}", s)))
    }
}
